"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.embedding_finetuning_method import EmbeddingFinetuningMethod
from ..models.pretrained_embedding_model_name import PretrainedEmbeddingModelName
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.finetune_embedding_model_request_training_args import FinetuneEmbeddingModelRequestTrainingArgs


T = TypeVar("T", bound="FinetuneEmbeddingModelRequest")


@_attrs_define
class FinetuneEmbeddingModelRequest:
    """
    Attributes:
        name (str):
        base_model (PretrainedEmbeddingModelName): Names of pretrained embedding models that are supported by OrcaCloud
        train_memoryset_id (Union[None, Unset, str]):
        train_memoryset_name (Union[None, Unset, str]):
        train_datasource_id (Union[None, Unset, str]):
        train_datasource_name (Union[None, Unset, str]):
        eval_datasource_id (Union[None, Unset, str]):
        eval_datasource_name (Union[None, Unset, str]):
        label_column (Union[Unset, str]):  Default: 'label'.
        value_column (Union[Unset, str]):  Default: 'value'.
        training_method (Union[Unset, EmbeddingFinetuningMethod]):
        training_args (Union[Unset, FinetuneEmbeddingModelRequestTrainingArgs]):
    """

    name: str
    base_model: PretrainedEmbeddingModelName
    train_memoryset_id: Union[None, Unset, str] = UNSET
    train_memoryset_name: Union[None, Unset, str] = UNSET
    train_datasource_id: Union[None, Unset, str] = UNSET
    train_datasource_name: Union[None, Unset, str] = UNSET
    eval_datasource_id: Union[None, Unset, str] = UNSET
    eval_datasource_name: Union[None, Unset, str] = UNSET
    label_column: Union[Unset, str] = "label"
    value_column: Union[Unset, str] = "value"
    training_method: Union[Unset, EmbeddingFinetuningMethod] = UNSET
    training_args: Union[Unset, "FinetuneEmbeddingModelRequestTrainingArgs"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        base_model = self.base_model.value if isinstance(self.base_model, Enum) else self.base_model

        train_memoryset_id: Union[None, Unset, str]
        if isinstance(self.train_memoryset_id, Unset):
            train_memoryset_id = UNSET
        else:
            train_memoryset_id = self.train_memoryset_id

        train_memoryset_name: Union[None, Unset, str]
        if isinstance(self.train_memoryset_name, Unset):
            train_memoryset_name = UNSET
        else:
            train_memoryset_name = self.train_memoryset_name

        train_datasource_id: Union[None, Unset, str]
        if isinstance(self.train_datasource_id, Unset):
            train_datasource_id = UNSET
        else:
            train_datasource_id = self.train_datasource_id

        train_datasource_name: Union[None, Unset, str]
        if isinstance(self.train_datasource_name, Unset):
            train_datasource_name = UNSET
        else:
            train_datasource_name = self.train_datasource_name

        eval_datasource_id: Union[None, Unset, str]
        if isinstance(self.eval_datasource_id, Unset):
            eval_datasource_id = UNSET
        else:
            eval_datasource_id = self.eval_datasource_id

        eval_datasource_name: Union[None, Unset, str]
        if isinstance(self.eval_datasource_name, Unset):
            eval_datasource_name = UNSET
        else:
            eval_datasource_name = self.eval_datasource_name

        label_column = self.label_column

        value_column = self.value_column

        training_method: Union[Unset, str] = UNSET
        if not isinstance(self.training_method, Unset):
            training_method = self.training_method.value

        training_args: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.training_args, Unset):
            training_args = self.training_args.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "base_model": base_model,
            }
        )
        if train_memoryset_id is not UNSET:
            field_dict["train_memoryset_id"] = train_memoryset_id
        if train_memoryset_name is not UNSET:
            field_dict["train_memoryset_name"] = train_memoryset_name
        if train_datasource_id is not UNSET:
            field_dict["train_datasource_id"] = train_datasource_id
        if train_datasource_name is not UNSET:
            field_dict["train_datasource_name"] = train_datasource_name
        if eval_datasource_id is not UNSET:
            field_dict["eval_datasource_id"] = eval_datasource_id
        if eval_datasource_name is not UNSET:
            field_dict["eval_datasource_name"] = eval_datasource_name
        if label_column is not UNSET:
            field_dict["label_column"] = label_column
        if value_column is not UNSET:
            field_dict["value_column"] = value_column
        if training_method is not UNSET:
            field_dict["training_method"] = training_method
        if training_args is not UNSET:
            field_dict["training_args"] = training_args

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.finetune_embedding_model_request_training_args import FinetuneEmbeddingModelRequestTrainingArgs

        d = src_dict.copy()
        name = d.pop("name")

        base_model = PretrainedEmbeddingModelName(d.pop("base_model"))

        def _parse_train_memoryset_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        train_memoryset_id = _parse_train_memoryset_id(d.pop("train_memoryset_id", UNSET))

        def _parse_train_memoryset_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        train_memoryset_name = _parse_train_memoryset_name(d.pop("train_memoryset_name", UNSET))

        def _parse_train_datasource_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        train_datasource_id = _parse_train_datasource_id(d.pop("train_datasource_id", UNSET))

        def _parse_train_datasource_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        train_datasource_name = _parse_train_datasource_name(d.pop("train_datasource_name", UNSET))

        def _parse_eval_datasource_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        eval_datasource_id = _parse_eval_datasource_id(d.pop("eval_datasource_id", UNSET))

        def _parse_eval_datasource_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        eval_datasource_name = _parse_eval_datasource_name(d.pop("eval_datasource_name", UNSET))

        label_column = d.pop("label_column", UNSET)

        value_column = d.pop("value_column", UNSET)

        _training_method = d.pop("training_method", UNSET)
        training_method: Union[Unset, EmbeddingFinetuningMethod]
        if isinstance(_training_method, Unset):
            training_method = UNSET
        else:
            training_method = EmbeddingFinetuningMethod(_training_method)

        _training_args = d.pop("training_args", UNSET)
        training_args: Union[Unset, FinetuneEmbeddingModelRequestTrainingArgs]
        if isinstance(_training_args, Unset):
            training_args = UNSET
        else:
            training_args = FinetuneEmbeddingModelRequestTrainingArgs.from_dict(_training_args)

        finetune_embedding_model_request = cls(
            name=name,
            base_model=base_model,
            train_memoryset_id=train_memoryset_id,
            train_memoryset_name=train_memoryset_name,
            train_datasource_id=train_datasource_id,
            train_datasource_name=train_datasource_name,
            eval_datasource_id=eval_datasource_id,
            eval_datasource_name=eval_datasource_name,
            label_column=label_column,
            value_column=value_column,
            training_method=training_method,
            training_args=training_args,
        )

        finetune_embedding_model_request.additional_properties = d
        return finetune_embedding_model_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="MemorysetDuplicateMetrics")


@_attrs_define
class MemorysetDuplicateMetrics:
    """
    Attributes:
        num_duplicates (int):
    """

    num_duplicates: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        num_duplicates = self.num_duplicates

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "num_duplicates": num_duplicates,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        num_duplicates = d.pop("num_duplicates")

        memoryset_duplicate_metrics = cls(
            num_duplicates=num_duplicates,
        )

        memoryset_duplicate_metrics.additional_properties = d
        return memoryset_duplicate_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_item import FilterItem


T = TypeVar("T", bound="ListMemoriesRequest")


@_attrs_define
class ListMemoriesRequest:
    """
    Attributes:
        offset (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 100.
        filters (Union[Unset, List['FilterItem']]):
    """

    offset: Union[Unset, int] = 0
    limit: Union[Unset, int] = 100
    filters: Union[Unset, List["FilterItem"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        offset = self.offset

        limit = self.limit

        filters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item = filters_item_data.to_dict()
                filters.append(filters_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if offset is not UNSET:
            field_dict["offset"] = offset
        if limit is not UNSET:
            field_dict["limit"] = limit
        if filters is not UNSET:
            field_dict["filters"] = filters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.filter_item import FilterItem

        d = src_dict.copy()
        offset = d.pop("offset", UNSET)

        limit = d.pop("limit", UNSET)

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:
            filters_item = FilterItem.from_dict(filters_item_data)

            filters.append(filters_item)

        list_memories_request = cls(
            offset=offset,
            limit=limit,
            filters=filters,
        )

        list_memories_request.additional_properties = d
        return list_memories_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TaskStatus(str, Enum):
    ABORTED = "ABORTED"
    ABORTING = "ABORTING"
    COMPLETED = "COMPLETED"
    DISPATCHED = "DISPATCHED"
    FAILED = "FAILED"
    INITIALIZED = "INITIALIZED"
    PROCESSING = "PROCESSING"

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from enum import Enum
from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.task_status import TaskStatus

if TYPE_CHECKING:
    from ..models.base_model import BaseModel


T = TypeVar("T", bound="Task")


@_attrs_define
class Task:
    """
    Attributes:
        status (TaskStatus): Status of task in the task queue
        steps_total (Union[None, int]):
        steps_completed (Union[None, int]):
        exception (Union[None, str]):
        updated_at (datetime.datetime):
        created_at (datetime.datetime):
        id (str):
        org_id (str):
        type (str):
        payload (BaseModel):
        result (Union['BaseModel', None]):
    """

    status: TaskStatus
    steps_total: Union[None, int]
    steps_completed: Union[None, int]
    exception: Union[None, str]
    updated_at: datetime.datetime
    created_at: datetime.datetime
    id: str
    org_id: str
    type: str
    payload: "BaseModel"
    result: Union["BaseModel", None]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.base_model import BaseModel

        status = self.status.value if isinstance(self.status, Enum) else self.status

        steps_total: Union[None, int]
        steps_total = self.steps_total

        steps_completed: Union[None, int]
        steps_completed = self.steps_completed

        exception: Union[None, str]
        exception = self.exception

        updated_at = self.updated_at.isoformat()

        created_at = self.created_at.isoformat()

        id = self.id

        org_id = self.org_id

        type = self.type

        payload = self.payload.to_dict()

        result: Union[Dict[str, Any], None]
        if isinstance(self.result, BaseModel):
            result = self.result.to_dict()
        else:
            result = self.result

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "steps_total": steps_total,
                "steps_completed": steps_completed,
                "exception": exception,
                "updated_at": updated_at,
                "created_at": created_at,
                "id": id,
                "org_id": org_id,
                "type": type,
                "payload": payload,
                "result": result,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.base_model import BaseModel

        d = src_dict.copy()
        status = TaskStatus(d.pop("status"))

        def _parse_steps_total(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        steps_total = _parse_steps_total(d.pop("steps_total"))

        def _parse_steps_completed(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        steps_completed = _parse_steps_completed(d.pop("steps_completed"))

        def _parse_exception(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        exception = _parse_exception(d.pop("exception"))

        updated_at = isoparse(d.pop("updated_at"))

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id")

        org_id = d.pop("org_id")

        type = d.pop("type")

        payload = BaseModel.from_dict(d.pop("payload"))

        def _parse_result(data: object) -> Union["BaseModel", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                result_type_0 = BaseModel.from_dict(data)

                return result_type_0
            except:  # noqa: E722
                pass
            return cast(Union["BaseModel", None], data)

        result = _parse_result(d.pop("result"))

        task = cls(
            status=status,
            steps_total=steps_total,
            steps_completed=steps_completed,
            exception=exception,
            updated_at=updated_at,
            created_at=created_at,
            id=id,
            org_id=org_id,
            type=type,
            payload=payload,
            result=result,
        )

        task.additional_properties = d
        return task

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PredictionFeedbackRequest")


@_attrs_define
class PredictionFeedbackRequest:
    """
    Attributes:
        prediction_id (str):
        category_name (str):
        value (Union[None, Unset, bool, float]): The feedback value. For updates, UNSET means keep existing value. None
            means delete the feedback.
        comment (Union[None, Unset, str]): Optional comment. For updates, UNSET means keep existing comment. None means
            remove the comment.
    """

    prediction_id: str
    category_name: str
    value: Union[None, Unset, bool, float] = UNSET
    comment: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prediction_id = self.prediction_id

        category_name = self.category_name

        value: Union[None, Unset, bool, float]
        if isinstance(self.value, Unset):
            value = UNSET
        else:
            value = self.value

        comment: Union[None, Unset, str]
        if isinstance(self.comment, Unset):
            comment = UNSET
        else:
            comment = self.comment

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prediction_id": prediction_id,
                "category_name": category_name,
            }
        )
        if value is not UNSET:
            field_dict["value"] = value
        if comment is not UNSET:
            field_dict["comment"] = comment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        prediction_id = d.pop("prediction_id")

        category_name = d.pop("category_name")

        def _parse_value(data: object) -> Union[None, Unset, bool, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool, float], data)

        value = _parse_value(d.pop("value", UNSET))

        def _parse_comment(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        comment = _parse_comment(d.pop("comment", UNSET))

        prediction_feedback_request = cls(
            prediction_id=prediction_id,
            category_name=category_name,
            value=value,
            comment=comment,
        )

        prediction_feedback_request.additional_properties = d
        return prediction_feedback_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="MemorysetProjectionMetrics")


@_attrs_define
class MemorysetProjectionMetrics:
    """ """

    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        memoryset_projection_metrics = cls()

        memoryset_projection_metrics.additional_properties = d
        return memoryset_projection_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .analyze_neighbor_labels_result import AnalyzeNeighborLabelsResult
from .api_key_metadata import ApiKeyMetadata
from .api_key_metadata_scope_item import ApiKeyMetadataScopeItem
from .base_model import BaseModel
from .body_create_datasource_datasource_post import BodyCreateDatasourceDatasourcePost
from .classification_evaluation_result import ClassificationEvaluationResult
from .clone_labeled_memoryset_request import CloneLabeledMemorysetRequest
from .cluster_metrics import ClusterMetrics
from .column_info import ColumnInfo
from .column_type import ColumnType
from .constraint_violation_error_response import ConstraintViolationErrorResponse
from .create_api_key_request import CreateApiKeyRequest
from .create_api_key_request_scope_item import CreateApiKeyRequestScopeItem
from .create_api_key_response import CreateApiKeyResponse
from .create_api_key_response_scope_item import CreateApiKeyResponseScopeItem
from .create_labeled_memoryset_request import CreateLabeledMemorysetRequest
from .create_rac_model_request import CreateRACModelRequest
from .datasource_metadata import DatasourceMetadata
from .delete_memories_request import DeleteMemoriesRequest
from .embed_request import EmbedRequest
from .embedding_evaluation_request import EmbeddingEvaluationRequest
from .embedding_evaluation_response import EmbeddingEvaluationResponse
from .embedding_evaluation_result import EmbeddingEvaluationResult
from .embedding_finetuning_method import EmbeddingFinetuningMethod
from .embedding_model_result import EmbeddingModelResult
from .evaluation_request import EvaluationRequest
from .evaluation_response import EvaluationResponse
from .feedback_metrics import FeedbackMetrics
from .feedback_type import FeedbackType
from .filter_item import FilterItem
from .filter_item_field_type_0_item import FilterItemFieldType0Item
from .filter_item_field_type_2_item_type_1 import FilterItemFieldType2ItemType1
from .filter_item_op import FilterItemOp
from .finetune_embedding_model_request import FinetuneEmbeddingModelRequest
from .finetune_embedding_model_request_training_args import FinetuneEmbeddingModelRequestTrainingArgs
from .finetuned_embedding_model_metadata import FinetunedEmbeddingModelMetadata
from .get_memories_request import GetMemoriesRequest
from .internal_server_error_response import InternalServerErrorResponse
from .label_class_metrics import LabelClassMetrics
from .label_prediction_memory_lookup import LabelPredictionMemoryLookup
from .label_prediction_memory_lookup_metadata import LabelPredictionMemoryLookupMetadata
from .label_prediction_result import LabelPredictionResult
from .label_prediction_with_memories_and_feedback import LabelPredictionWithMemoriesAndFeedback
from .labeled_memory import LabeledMemory
from .labeled_memory_insert import LabeledMemoryInsert
from .labeled_memory_insert_metadata import LabeledMemoryInsertMetadata
from .labeled_memory_lookup import LabeledMemoryLookup
from .labeled_memory_lookup_metadata import LabeledMemoryLookupMetadata
from .labeled_memory_metadata import LabeledMemoryMetadata
from .labeled_memory_metrics import LabeledMemoryMetrics
from .labeled_memory_update import LabeledMemoryUpdate
from .labeled_memory_update_metadata_type_0 import LabeledMemoryUpdateMetadataType0
from .labeled_memory_with_feedback_metrics import LabeledMemoryWithFeedbackMetrics
from .labeled_memory_with_feedback_metrics_feedback_metrics import LabeledMemoryWithFeedbackMetricsFeedbackMetrics
from .labeled_memory_with_feedback_metrics_metadata import LabeledMemoryWithFeedbackMetricsMetadata
from .labeled_memoryset_metadata import LabeledMemorysetMetadata
from .list_memories_request import ListMemoriesRequest
from .list_predictions_request import ListPredictionsRequest
from .lookup_request import LookupRequest
from .lookup_score_metrics import LookupScoreMetrics
from .memory_metrics import MemoryMetrics
from .memoryset_analysis_configs import MemorysetAnalysisConfigs
from .memoryset_analysis_request import MemorysetAnalysisRequest
from .memoryset_analysis_response import MemorysetAnalysisResponse
from .memoryset_cluster_analysis_config import MemorysetClusterAnalysisConfig
from .memoryset_cluster_analysis_config_clustering_method import MemorysetClusterAnalysisConfigClusteringMethod
from .memoryset_cluster_analysis_config_partitioning_method import MemorysetClusterAnalysisConfigPartitioningMethod
from .memoryset_cluster_metrics import MemorysetClusterMetrics
from .memoryset_duplicate_analysis_config import MemorysetDuplicateAnalysisConfig
from .memoryset_duplicate_metrics import MemorysetDuplicateMetrics
from .memoryset_label_analysis_config import MemorysetLabelAnalysisConfig
from .memoryset_label_metrics import MemorysetLabelMetrics
from .memoryset_metrics import MemorysetMetrics
from .memoryset_neighbor_analysis_config import MemorysetNeighborAnalysisConfig
from .memoryset_neighbor_metrics import MemorysetNeighborMetrics
from .memoryset_neighbor_metrics_lookup_score_metrics import MemorysetNeighborMetricsLookupScoreMetrics
from .memoryset_projection_analysis_config import MemorysetProjectionAnalysisConfig
from .memoryset_projection_metrics import MemorysetProjectionMetrics
from .not_found_error_response import NotFoundErrorResponse
from .not_found_error_response_resource_type_0 import NotFoundErrorResponseResourceType0
from .paginated_labeled_memory_with_feedback_metrics import PaginatedLabeledMemoryWithFeedbackMetrics
from .precision_recall_curve import PrecisionRecallCurve
from .prediction_feedback import PredictionFeedback
from .prediction_feedback_category import PredictionFeedbackCategory
from .prediction_feedback_request import PredictionFeedbackRequest
from .prediction_feedback_result import PredictionFeedbackResult
from .prediction_request import PredictionRequest
from .prediction_sort_item_item_type_0 import PredictionSortItemItemType0
from .prediction_sort_item_item_type_1 import PredictionSortItemItemType1
from .pretrained_embedding_model_metadata import PretrainedEmbeddingModelMetadata
from .pretrained_embedding_model_name import PretrainedEmbeddingModelName
from .rac_head_type import RACHeadType
from .rac_model_metadata import RACModelMetadata
from .roc_curve import ROCCurve
from .service_unavailable_error_response import ServiceUnavailableErrorResponse
from .task import Task
from .task_status import TaskStatus
from .task_status_info import TaskStatusInfo
from .telemetry_field_type_0_item_type_2 import TelemetryFieldType0ItemType2
from .telemetry_filter_item import TelemetryFilterItem
from .telemetry_filter_item_op import TelemetryFilterItemOp
from .telemetry_memories_request import TelemetryMemoriesRequest
from .telemetry_sort_options import TelemetrySortOptions
from .telemetry_sort_options_direction import TelemetrySortOptionsDirection
from .unauthenticated_error_response import UnauthenticatedErrorResponse
from .unauthorized_error_response import UnauthorizedErrorResponse
from .update_prediction_request import UpdatePredictionRequest

__all__ = (
    "AnalyzeNeighborLabelsResult",
    "ApiKeyMetadata",
    "ApiKeyMetadataScopeItem",
    "BaseModel",
    "BodyCreateDatasourceDatasourcePost",
    "ClassificationEvaluationResult",
    "CloneLabeledMemorysetRequest",
    "ClusterMetrics",
    "ColumnInfo",
    "ColumnType",
    "ConstraintViolationErrorResponse",
    "CreateApiKeyRequest",
    "CreateApiKeyRequestScopeItem",
    "CreateApiKeyResponse",
    "CreateApiKeyResponseScopeItem",
    "CreateLabeledMemorysetRequest",
    "CreateRACModelRequest",
    "DatasourceMetadata",
    "DeleteMemoriesRequest",
    "EmbeddingEvaluationRequest",
    "EmbeddingEvaluationResponse",
    "EmbeddingEvaluationResult",
    "EmbeddingFinetuningMethod",
    "EmbeddingModelResult",
    "EmbedRequest",
    "EvaluationRequest",
    "EvaluationResponse",
    "FeedbackMetrics",
    "FeedbackType",
    "FilterItem",
    "FilterItemFieldType0Item",
    "FilterItemFieldType2ItemType1",
    "FilterItemOp",
    "FinetunedEmbeddingModelMetadata",
    "FinetuneEmbeddingModelRequest",
    "FinetuneEmbeddingModelRequestTrainingArgs",
    "GetMemoriesRequest",
    "InternalServerErrorResponse",
    "LabelClassMetrics",
    "LabeledMemory",
    "LabeledMemoryInsert",
    "LabeledMemoryInsertMetadata",
    "LabeledMemoryLookup",
    "LabeledMemoryLookupMetadata",
    "LabeledMemoryMetadata",
    "LabeledMemoryMetrics",
    "LabeledMemorysetMetadata",
    "LabeledMemoryUpdate",
    "LabeledMemoryUpdateMetadataType0",
    "LabeledMemoryWithFeedbackMetrics",
    "LabeledMemoryWithFeedbackMetricsFeedbackMetrics",
    "LabeledMemoryWithFeedbackMetricsMetadata",
    "LabelPredictionMemoryLookup",
    "LabelPredictionMemoryLookupMetadata",
    "LabelPredictionResult",
    "LabelPredictionWithMemoriesAndFeedback",
    "ListMemoriesRequest",
    "ListPredictionsRequest",
    "LookupRequest",
    "LookupScoreMetrics",
    "MemoryMetrics",
    "MemorysetAnalysisConfigs",
    "MemorysetAnalysisRequest",
    "MemorysetAnalysisResponse",
    "MemorysetClusterAnalysisConfig",
    "MemorysetClusterAnalysisConfigClusteringMethod",
    "MemorysetClusterAnalysisConfigPartitioningMethod",
    "MemorysetClusterMetrics",
    "MemorysetDuplicateAnalysisConfig",
    "MemorysetDuplicateMetrics",
    "MemorysetLabelAnalysisConfig",
    "MemorysetLabelMetrics",
    "MemorysetMetrics",
    "MemorysetNeighborAnalysisConfig",
    "MemorysetNeighborMetrics",
    "MemorysetNeighborMetricsLookupScoreMetrics",
    "MemorysetProjectionAnalysisConfig",
    "MemorysetProjectionMetrics",
    "NotFoundErrorResponse",
    "NotFoundErrorResponseResourceType0",
    "PaginatedLabeledMemoryWithFeedbackMetrics",
    "PrecisionRecallCurve",
    "PredictionFeedback",
    "PredictionFeedbackCategory",
    "PredictionFeedbackRequest",
    "PredictionFeedbackResult",
    "PredictionRequest",
    "PredictionSortItemItemType0",
    "PredictionSortItemItemType1",
    "PretrainedEmbeddingModelMetadata",
    "PretrainedEmbeddingModelName",
    "RACHeadType",
    "RACModelMetadata",
    "ROCCurve",
    "ServiceUnavailableErrorResponse",
    "Task",
    "TaskStatus",
    "TaskStatusInfo",
    "TelemetryFieldType0ItemType2",
    "TelemetryFilterItem",
    "TelemetryFilterItemOp",
    "TelemetryMemoriesRequest",
    "TelemetrySortOptions",
    "TelemetrySortOptionsDirection",
    "UnauthenticatedErrorResponse",
    "UnauthorizedErrorResponse",
    "UpdatePredictionRequest",
)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LabeledMemoryLookupMetadata")


@_attrs_define
class LabeledMemoryLookupMetadata:
    """ """

    additional_properties: dict[str, Union[None, bool, float, int, str]] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_dict: dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        labeled_memory_lookup_metadata = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(data: object) -> Union[None, bool, float, int, str]:
                if data is None:
                    return data
                return cast(Union[None, bool, float, int, str], data)

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        labeled_memory_lookup_metadata.additional_properties = additional_properties
        return labeled_memory_lookup_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Union[None, bool, float, int, str]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Union[None, bool, float, int, str]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.label_class_metrics import LabelClassMetrics


T = TypeVar("T", bound="AnalyzeNeighborLabelsResult")


@_attrs_define
class AnalyzeNeighborLabelsResult:
    """
    Attributes:
        label_metrics (List['LabelClassMetrics']):
        neighbor_prediction_accuracy (float):
        mean_neighbor_label_confidence (float):
        mean_neighbor_label_entropy (float):
        mean_neighbor_predicted_label_ambiguity (float):
    """

    label_metrics: List["LabelClassMetrics"]
    neighbor_prediction_accuracy: float
    mean_neighbor_label_confidence: float
    mean_neighbor_label_entropy: float
    mean_neighbor_predicted_label_ambiguity: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label_metrics = []
        for label_metrics_item_data in self.label_metrics:
            label_metrics_item = label_metrics_item_data.to_dict()
            label_metrics.append(label_metrics_item)

        neighbor_prediction_accuracy = self.neighbor_prediction_accuracy

        mean_neighbor_label_confidence = self.mean_neighbor_label_confidence

        mean_neighbor_label_entropy = self.mean_neighbor_label_entropy

        mean_neighbor_predicted_label_ambiguity = self.mean_neighbor_predicted_label_ambiguity

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "label_metrics": label_metrics,
                "neighbor_prediction_accuracy": neighbor_prediction_accuracy,
                "mean_neighbor_label_confidence": mean_neighbor_label_confidence,
                "mean_neighbor_label_entropy": mean_neighbor_label_entropy,
                "mean_neighbor_predicted_label_ambiguity": mean_neighbor_predicted_label_ambiguity,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.label_class_metrics import LabelClassMetrics

        d = src_dict.copy()
        label_metrics = []
        _label_metrics = d.pop("label_metrics")
        for label_metrics_item_data in _label_metrics:
            label_metrics_item = LabelClassMetrics.from_dict(label_metrics_item_data)

            label_metrics.append(label_metrics_item)

        neighbor_prediction_accuracy = d.pop("neighbor_prediction_accuracy")

        mean_neighbor_label_confidence = d.pop("mean_neighbor_label_confidence")

        mean_neighbor_label_entropy = d.pop("mean_neighbor_label_entropy")

        mean_neighbor_predicted_label_ambiguity = d.pop("mean_neighbor_predicted_label_ambiguity")

        analyze_neighbor_labels_result = cls(
            label_metrics=label_metrics,
            neighbor_prediction_accuracy=neighbor_prediction_accuracy,
            mean_neighbor_label_confidence=mean_neighbor_label_confidence,
            mean_neighbor_label_entropy=mean_neighbor_label_entropy,
            mean_neighbor_predicted_label_ambiguity=mean_neighbor_predicted_label_ambiguity,
        )

        analyze_neighbor_labels_result.additional_properties = d
        return analyze_neighbor_labels_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LookupScoreMetrics")


@_attrs_define
class LookupScoreMetrics:
    """
    Attributes:
        median (float):
        std (float):
        quantiles (List[float]):
        quantile_values (List[float]):
    """

    median: float
    std: float
    quantiles: List[float]
    quantile_values: List[float]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        median = self.median

        std = self.std

        quantiles = self.quantiles

        quantile_values = self.quantile_values

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "median": median,
                "std": std,
                "quantiles": quantiles,
                "quantile_values": quantile_values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        median = d.pop("median")

        std = d.pop("std")

        quantiles = cast(List[float], d.pop("quantiles"))

        quantile_values = cast(List[float], d.pop("quantile_values"))

        lookup_score_metrics = cls(
            median=median,
            std=std,
            quantiles=quantiles,
            quantile_values=quantile_values,
        )

        lookup_score_metrics.additional_properties = d
        return lookup_score_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdatePredictionRequest")


@_attrs_define
class UpdatePredictionRequest:
    """
    Attributes:
        expected_label (Union[None, Unset, int]):
        tags (Union[Unset, List[str]]):
    """

    expected_label: Union[None, Unset, int] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        expected_label: Union[None, Unset, int]
        if isinstance(self.expected_label, Unset):
            expected_label = UNSET
        else:
            expected_label = self.expected_label

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if expected_label is not UNSET:
            field_dict["expected_label"] = expected_label
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_expected_label(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        expected_label = _parse_expected_label(d.pop("expected_label", UNSET))

        tags = cast(List[str], d.pop("tags", UNSET))

        update_prediction_request = cls(
            expected_label=expected_label,
            tags=tags,
        )

        update_prediction_request.additional_properties = d
        return update_prediction_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/5cfe4e1d594951725844ea470fc9d61f40c08093/openapi_python_client/templates/client.py.jinja

The main changes are:
- add a lazy initialized global client singleton that is injected into every api method via a context manager
- add methods for setting headers and base URL on the global client singleton
"""

import os
import ssl
from contextlib import contextmanager
from typing import Any, Optional, Union

import httpx
from attrs import define, evolve, field


@define
class Client:
    """A class for keeping track of data related to the API

    The following are accepted as keyword arguments and will be used to construct httpx Clients internally:

        ``base_url``: The base URL for the API, all requests are made to a relative path to this URL

        ``cookies``: A dictionary of cookies to be sent with every request

        ``headers``: A dictionary of headers to be sent with every request

        ``timeout``: The maximum amount of a time a request can take. API functions will raise
        httpx.TimeoutException if this is exceeded.

        ``verify_ssl``: Whether or not to verify the SSL certificate of the API server. This should be True in production,
        but can be set to False for testing purposes.

        ``follow_redirects``: Whether or not to follow redirects. Default value is False.

        ``httpx_args``: A dictionary of additional arguments to be passed to the ``httpx.Client`` and ``httpx.AsyncClient`` constructor.


    Attributes:
        raise_on_unexpected_status: Whether or not to raise an errors.UnexpectedStatus if the API returns a
            status code that was not documented in the source OpenAPI document. Can also be provided as a keyword
            argument to the constructor.
    """

    raise_on_unexpected_status: bool = field(default=False, kw_only=True)
    _base_url: str = field(alias="base_url")
    _cookies: dict[str, str] = field(factory=dict, kw_only=True, alias="cookies")
    _headers: dict[str, str] = field(factory=dict, kw_only=True, alias="headers")
    _timeout: Optional[httpx.Timeout] = field(default=None, kw_only=True, alias="timeout")
    _verify_ssl: Union[str, bool, ssl.SSLContext] = field(default=True, kw_only=True, alias="verify_ssl")
    _follow_redirects: bool = field(default=False, kw_only=True, alias="follow_redirects")
    _httpx_args: dict[str, Any] = field(factory=dict, kw_only=True, alias="httpx_args")
    _client: Optional[httpx.Client] = field(default=None, init=False)
    _async_client: Optional[httpx.AsyncClient] = field(default=None, init=False)

    def with_headers(self, headers: dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        if self._client is not None:
            self._client.headers.update(headers)
        if self._async_client is not None:
            self._async_client.headers.update(headers)
        return evolve(self, headers={**self._headers, **headers})

    def with_cookies(self, cookies: dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        if self._client is not None:
            self._client.cookies.update(cookies)
        if self._async_client is not None:
            self._async_client.cookies.update(cookies)
        return evolve(self, cookies={**self._cookies, **cookies})

    def with_timeout(self, timeout: httpx.Timeout) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        if self._client is not None:
            self._client.timeout = timeout
        if self._async_client is not None:
            self._async_client.timeout = timeout
        return evolve(self, timeout=timeout)

    def set_httpx_client(self, client: httpx.Client) -> "Client":
        """Manually set the underlying httpx.Client

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._client = client
        return self

    def get_httpx_client(self) -> httpx.Client:
        """Get the underlying httpx.Client, constructing a new one if not previously set"""
        if self._client is None:
            self._client = httpx.Client(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._client

    def __enter__(self) -> "Client":
        """Enter a context manager for self.client—you cannot enter twice (see httpx docs)"""
        self.get_httpx_client().__enter__()
        return self

    def __exit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for internal httpx.Client (see httpx docs)"""
        self.get_httpx_client().__exit__(*args, **kwargs)

    def set_async_httpx_client(self, async_client: httpx.AsyncClient) -> "Client":
        """Manually the underlying httpx.AsyncClient

        **NOTE**: This will override any other settings on the client, including cookies, headers, and timeout.
        """
        self._async_client = async_client
        return self

    def get_async_httpx_client(self) -> httpx.AsyncClient:
        """Get the underlying httpx.AsyncClient, constructing a new one if not previously set"""
        if self._async_client is None:
            self._async_client = httpx.AsyncClient(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._async_client

    async def __aenter__(self) -> "Client":
        """Enter a context manager for underlying httpx.AsyncClient—you cannot enter twice (see httpx docs)"""
        await self.get_async_httpx_client().__aenter__()
        return self

    async def __aexit__(self, *args: Any, **kwargs: Any) -> None:
        """Exit a context manager for underlying httpx.AsyncClient (see httpx docs)"""
        await self.get_async_httpx_client().__aexit__(*args, **kwargs)


# this must be initialized lazily to ensure that the environment variables are set
_client: Optional["Client"] = None


def get_base_url():
    """Get the base URL for the global client singleton"""
    global _client
    if _client is not None:
        return _client._base_url
    return os.environ.get("ORCA_API_URL", "https://api.orcadb.ai/")


def get_headers():
    """Get the headers for the global client singleton"""
    global _client
    if _client is not None:
        return _client._headers
    headers = {}
    api_key = os.environ.get("ORCA_API_KEY")
    if api_key is not None:
        headers["Api-Key"] = api_key
    return headers


def get_client():
    """Get the global client singleton"""
    global _client
    if _client is None:
        _client = Client(base_url=get_base_url(), headers=get_headers())
    return _client


def set_headers(headers: dict[str, str]):
    """Set the headers for the global client singleton"""
    global _client
    _client = Client(base_url=get_base_url(), headers=headers)
    return _client


def set_base_url(base_url: str):
    """Set the base URL for the global client singleton"""
    global _client
    _client = Client(base_url=base_url, headers=_client._headers if _client is not None else {})
    return _client


@contextmanager
def headers_context(headers: dict[str, str]):
    """Set the headers for the global client singleton within a context manager"""
    global _client
    old_client = get_client()
    set_headers(headers=headers)
    yield
    _client = old_client


@contextmanager
def base_url_context(base_url: str):
    """Set the base URL for the global client singleton within a context manager"""
    global _client
    old_client = get_client()
    set_base_url(base_url=base_url)
    yield
    _client = old_client


@contextmanager
def _client_context():
    yield get_client()



"""
OrcaSDK is a Python library for building and using retrieval augmented models in the OrcaCloud.
"""

from ._utils.common import UNSET, CreateMode, DropMode
from ._utils.task import TaskStatus
from .classification_model import ClassificationModel
from .credentials import OrcaCredentials
from .datasource import Datasource
from .embedding_model import (
    FinetunedEmbeddingModel,
    PretrainedEmbeddingModel,
    PretrainedEmbeddingModelName,
)
from .memoryset import (
    FilterItemTuple,
    LabeledMemory,
    LabeledMemoryLookup,
    LabeledMemoryset,
)
from .telemetry import LabelPrediction

# only specify things that should show up on the root page of the reference docs because they are in private modules
__all__ = ["TaskStatus", "UNSET", "CreateMode", "DropMode"]



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_api_key_request_scope_item import CreateApiKeyRequestScopeItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateApiKeyRequest")


@_attrs_define
class CreateApiKeyRequest:
    """
    Attributes:
        scope (List[CreateApiKeyRequestScopeItem]):
        id (Union[Unset, str]):
        name (Union[Unset, str]):  Default: 'default'.
        created_by (Union[None, Unset, str]):
    """

    scope: List[CreateApiKeyRequestScopeItem]
    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = "default"
    created_by: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        scope = []
        for scope_item_data in self.scope:
            scope_item = scope_item_data.value if isinstance(scope_item_data, Enum) else scope_item_data
            scope.append(scope_item)

        id = self.id

        name = self.name

        created_by: Union[None, Unset, str]
        if isinstance(self.created_by, Unset):
            created_by = UNSET
        else:
            created_by = self.created_by

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "scope": scope,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if created_by is not UNSET:
            field_dict["created_by"] = created_by

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        scope = []
        _scope = d.pop("scope")
        for scope_item_data in _scope:
            scope_item = CreateApiKeyRequestScopeItem(scope_item_data)

            scope.append(scope_item)

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_created_by(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        created_by = _parse_created_by(d.pop("created_by", UNSET))

        create_api_key_request = cls(
            scope=scope,
            id=id,
            name=name,
            created_by=created_by,
        )

        create_api_key_request.additional_properties = d
        return create_api_key_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
import os
from typing import Generator
from uuid import uuid4

import pytest
from datasets import ClassLabel, Dataset, Features, Value

from ._utils.auth import _create_api_key, _delete_org
from .classification_model import ClassificationModel
from .credentials import OrcaCredentials
from .datasource import Datasource
from .embedding_model import PretrainedEmbeddingModel
from .memoryset import LabeledMemoryset

logging.basicConfig(level=logging.INFO)

os.environ["ORCA_API_URL"] = os.environ.get("ORCA_API_URL", "http://localhost:1584/")


def _create_org_id():
    # UUID start to identify test data (0xtest...)
    return "10e50000-0000-4000-a000-" + str(uuid4())[24:]


@pytest.fixture(scope="session")
def org_id():
    return _create_org_id()


@pytest.fixture(autouse=True, scope="session")
def api_key(org_id) -> Generator[str, None, None]:
    api_key = _create_api_key(org_id=org_id, name="orca_sdk_test")
    OrcaCredentials.set_api_key(api_key, check_validity=True)
    yield api_key
    _delete_org(org_id)


@pytest.fixture(autouse=True)
def authenticated(api_key):
    OrcaCredentials.set_api_key(api_key, check_validity=False)


@pytest.fixture()
def unauthenticated(api_key):
    OrcaCredentials.set_api_key(str(uuid4()), check_validity=False)
    yield
    # Need to reset the api key to the original api key so following tests don't fail
    OrcaCredentials.set_api_key(api_key, check_validity=False)


@pytest.fixture()
def other_org_id():
    return _create_org_id()


@pytest.fixture()
def unauthorized(api_key, other_org_id):
    different_api_key = _create_api_key(org_id=other_org_id, name="orca_sdk_test_other_org")
    OrcaCredentials.set_api_key(different_api_key, check_validity=False)
    yield
    OrcaCredentials.set_api_key(api_key, check_validity=False)
    _delete_org(other_org_id)


@pytest.fixture(scope="session")
def label_names():
    return ["soup", "cats"]


SAMPLE_DATA = [
    {"text": "i love soup", "label": 0, "key": "val1", "score": 0.1, "source_id": "s1"},
    {"text": "cats are cute", "label": 1, "key": "val2", "score": 0.2, "source_id": "s2"},
    {"text": "soup is good", "label": 0, "key": "val3", "score": 0.3, "source_id": "s3"},
    {"text": "i love cats", "label": 1, "key": "val4", "score": 0.4, "source_id": "s4"},
    {"text": "everyone loves cats", "label": 1, "key": "val5", "score": 0.5, "source_id": "s5"},
    {"text": "soup is great for the winter", "label": 0, "key": "val6", "score": 0.6, "source_id": "s6"},
]


@pytest.fixture(scope="session")
def hf_dataset(label_names):
    return Dataset.from_list(
        SAMPLE_DATA,
        features=Features(
            {
                "text": Value("string"),
                "label": ClassLabel(names=label_names),
                "key": Value("string"),
                "score": Value("float"),
                "source_id": Value("string"),
            }
        ),
    )


@pytest.fixture(scope="session")
def datasource(hf_dataset) -> Datasource:
    return Datasource.from_hf_dataset("test_datasource", hf_dataset)


@pytest.fixture(scope="session")
def memoryset(datasource) -> LabeledMemoryset:
    return LabeledMemoryset.create(
        "test_memoryset",
        datasource=datasource,
        embedding_model=PretrainedEmbeddingModel.GTE_BASE,
        value_column="text",
        source_id_column="source_id",
        max_seq_length_override=32,
    )


@pytest.fixture(scope="session")
def model(memoryset) -> ClassificationModel:
    return ClassificationModel.create("test_model", memoryset, num_classes=2, memory_lookup_count=3)



import base64
import io
from typing import cast

import numpy as np
from numpy.typing import NDArray
from PIL import Image as pil

ValueType = str | pil.Image | NDArray[np.float32]
"""
The type of a value in a memoryset

- `str`: string
- `pil.Image`: image
- `NDArray[np.float32]`: univariate or multivariate timeseries
"""


def decode_value(value: str) -> ValueType:
    if value.startswith("data:image"):
        header, data = value.split(",", 1)
        return pil.open(io.BytesIO(base64.b64decode(data)))

    if value.startswith("data:numpy"):
        header, data = value.split(",", 1)
        return np.load(io.BytesIO(base64.b64decode(data)))

    return value


def encode_value(value: ValueType) -> str:
    if isinstance(value, pil.Image):
        header = f"data:image/{value.format.lower()};base64," if value.format else "data:image;base64,"
        buffer = io.BytesIO()
        value.save(buffer, format=value.format)
        bytes = buffer.getvalue()
        return header + base64.b64encode(bytes).decode("utf-8")

    if isinstance(value, np.ndarray):
        header = f"data:numpy/{value.dtype.name};base64,"
        buffer = io.BytesIO()
        np.save(buffer, value)
        return header + base64.b64encode(buffer.getvalue()).decode("utf-8")

    return value



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.analyze_neighbor_labels_result import AnalyzeNeighborLabelsResult


T = TypeVar("T", bound="EmbeddingModelResult")


@_attrs_define
class EmbeddingModelResult:
    """
    Attributes:
        embedding_model_name (str):
        embedding_model_path (str):
        analysis_result (AnalyzeNeighborLabelsResult):
        memoryset_name (Union[None, Unset, str]):
    """

    embedding_model_name: str
    embedding_model_path: str
    analysis_result: "AnalyzeNeighborLabelsResult"
    memoryset_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        embedding_model_name = self.embedding_model_name

        embedding_model_path = self.embedding_model_path

        analysis_result = self.analysis_result.to_dict()

        memoryset_name: Union[None, Unset, str]
        if isinstance(self.memoryset_name, Unset):
            memoryset_name = UNSET
        else:
            memoryset_name = self.memoryset_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "embedding_model_name": embedding_model_name,
                "embedding_model_path": embedding_model_path,
                "analysis_result": analysis_result,
            }
        )
        if memoryset_name is not UNSET:
            field_dict["memoryset_name"] = memoryset_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.analyze_neighbor_labels_result import AnalyzeNeighborLabelsResult

        d = src_dict.copy()
        embedding_model_name = d.pop("embedding_model_name")

        embedding_model_path = d.pop("embedding_model_path")

        analysis_result = AnalyzeNeighborLabelsResult.from_dict(d.pop("analysis_result"))

        def _parse_memoryset_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        memoryset_name = _parse_memoryset_name(d.pop("memoryset_name", UNSET))

        embedding_model_result = cls(
            embedding_model_name=embedding_model_name,
            embedding_model_path=embedding_model_path,
            analysis_result=analysis_result,
            memoryset_name=memoryset_name,
        )

        embedding_model_result.additional_properties = d
        return embedding_model_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
import re
from pathlib import Path
from typing import TYPE_CHECKING

import gradio as gr

from ..memoryset import LabeledMemoryLookup

if TYPE_CHECKING:
    from ..telemetry import LabelPrediction


def inspect_prediction_result(prediction_result: "LabelPrediction"):
    label_names = prediction_result.memoryset.label_names

    def update_label(val: str, memory: LabeledMemoryLookup, progress=gr.Progress(track_tqdm=True)):
        progress(0)
        match = re.search(r".*\((\d+)\)$", val)
        if match:
            progress(0.5)
            new_label = int(match.group(1))
            memory.update(label=new_label)
            progress(1)
            return "&#9989; Changes saved"
        else:
            logging.error(f"Invalid label format: {val}")

    with gr.Blocks(
        fill_width=True,
        title="Prediction Results",
        css_paths=str(Path(__file__).parent / "prediction_result_ui.css"),
    ) as prediction_result_ui:
        gr.Markdown("# Prediction Results")
        gr.Markdown(f"**Input:** {prediction_result.input_value}")
        gr.Markdown(f"**Prediction:** {label_names[prediction_result.label]} ({prediction_result.label})")
        gr.Markdown("### Memory Lookups")

        with gr.Row(equal_height=True, variant="panel"):
            with gr.Column(scale=7):
                gr.Markdown("**Value**")
            with gr.Column(scale=3, min_width=150):
                gr.Markdown("**Label**")
        for i, mem_lookup in enumerate(prediction_result.memory_lookups):
            with gr.Row(equal_height=True, variant="panel", elem_classes="white" if i % 2 == 0 else None):
                with gr.Column(scale=7):
                    gr.Markdown(
                        (
                            mem_lookup.value
                            if isinstance(mem_lookup.value, str)
                            else "Time series data"
                            if isinstance(mem_lookup.value, list)
                            else "Image data"
                        ),
                        label="Value",
                        height=50,
                    )
                with gr.Column(scale=3, min_width=150):
                    dropdown = gr.Dropdown(
                        choices=[f"{label_name} ({i})" for i, label_name in enumerate(label_names)],
                        label="Label",
                        value=f"{label_names[mem_lookup.label]} ({mem_lookup.label})",
                        interactive=True,
                        container=False,
                    )
                    changes_saved = gr.HTML(lambda: "", elem_classes="success no-padding", every=15)
                    dropdown.change(
                        lambda val, mem_lookup=mem_lookup: update_label(val, mem_lookup),
                        inputs=[dropdown],
                        outputs=[changes_saved],
                        show_progress="full",
                    )

    prediction_result_ui.launch()



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EvaluationRequest")


@_attrs_define
class EvaluationRequest:
    """
    Attributes:
        datasource_label_column (str):
        datasource_value_column (str):
        datasource_id (Union[None, Unset, str]):
        datasource_name (Union[None, Unset, str]):
        memoryset_override_id (Union[None, Unset, str]):
        record_telemetry (Union[Unset, bool]):  Default: False.
        telemetry_tags (Union[List[str], None, Unset]):
    """

    datasource_label_column: str
    datasource_value_column: str
    datasource_id: Union[None, Unset, str] = UNSET
    datasource_name: Union[None, Unset, str] = UNSET
    memoryset_override_id: Union[None, Unset, str] = UNSET
    record_telemetry: Union[Unset, bool] = False
    telemetry_tags: Union[List[str], None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        datasource_label_column = self.datasource_label_column

        datasource_value_column = self.datasource_value_column

        datasource_id: Union[None, Unset, str]
        if isinstance(self.datasource_id, Unset):
            datasource_id = UNSET
        else:
            datasource_id = self.datasource_id

        datasource_name: Union[None, Unset, str]
        if isinstance(self.datasource_name, Unset):
            datasource_name = UNSET
        else:
            datasource_name = self.datasource_name

        memoryset_override_id: Union[None, Unset, str]
        if isinstance(self.memoryset_override_id, Unset):
            memoryset_override_id = UNSET
        else:
            memoryset_override_id = self.memoryset_override_id

        record_telemetry = self.record_telemetry

        telemetry_tags: Union[List[str], None, Unset]
        if isinstance(self.telemetry_tags, Unset):
            telemetry_tags = UNSET
        elif isinstance(self.telemetry_tags, list):
            telemetry_tags = self.telemetry_tags

        else:
            telemetry_tags = self.telemetry_tags

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "datasource_label_column": datasource_label_column,
                "datasource_value_column": datasource_value_column,
            }
        )
        if datasource_id is not UNSET:
            field_dict["datasource_id"] = datasource_id
        if datasource_name is not UNSET:
            field_dict["datasource_name"] = datasource_name
        if memoryset_override_id is not UNSET:
            field_dict["memoryset_override_id"] = memoryset_override_id
        if record_telemetry is not UNSET:
            field_dict["record_telemetry"] = record_telemetry
        if telemetry_tags is not UNSET:
            field_dict["telemetry_tags"] = telemetry_tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        datasource_label_column = d.pop("datasource_label_column")

        datasource_value_column = d.pop("datasource_value_column")

        def _parse_datasource_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        datasource_id = _parse_datasource_id(d.pop("datasource_id", UNSET))

        def _parse_datasource_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        datasource_name = _parse_datasource_name(d.pop("datasource_name", UNSET))

        def _parse_memoryset_override_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        memoryset_override_id = _parse_memoryset_override_id(d.pop("memoryset_override_id", UNSET))

        record_telemetry = d.pop("record_telemetry", UNSET)

        def _parse_telemetry_tags(data: object) -> Union[List[str], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                telemetry_tags_type_0 = cast(List[str], data)

                return telemetry_tags_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[str], None, Unset], data)

        telemetry_tags = _parse_telemetry_tags(d.pop("telemetry_tags", UNSET))

        evaluation_request = cls(
            datasource_label_column=datasource_label_column,
            datasource_value_column=datasource_value_column,
            datasource_id=datasource_id,
            datasource_name=datasource_name,
            memoryset_override_id=memoryset_override_id,
            record_telemetry=record_telemetry,
            telemetry_tags=telemetry_tags,
        )

        evaluation_request.additional_properties = d
        return evaluation_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from typing import TYPE_CHECKING, Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.labeled_memory_metrics import LabeledMemoryMetrics
    from ..models.labeled_memory_with_feedback_metrics_feedback_metrics import (
        LabeledMemoryWithFeedbackMetricsFeedbackMetrics,
    )
    from ..models.labeled_memory_with_feedback_metrics_metadata import LabeledMemoryWithFeedbackMetricsMetadata


T = TypeVar("T", bound="LabeledMemoryWithFeedbackMetrics")


@_attrs_define
class LabeledMemoryWithFeedbackMetrics:
    """
    Attributes:
        value (str):
        embedding (List[float]):
        source_id (Union[None, str]):
        metadata (LabeledMemoryWithFeedbackMetricsMetadata):
        memory_id (str):
        memory_version (int):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        metrics (LabeledMemoryMetrics): Metrics computed for a labeled memory.
        label (int):
        label_name (Union[None, str]):
        feedback_metrics (LabeledMemoryWithFeedbackMetricsFeedbackMetrics):
        lookup_count (int):
    """

    value: str
    embedding: List[float]
    source_id: Union[None, str]
    metadata: "LabeledMemoryWithFeedbackMetricsMetadata"
    memory_id: str
    memory_version: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    metrics: "LabeledMemoryMetrics"
    label: int
    label_name: Union[None, str]
    feedback_metrics: "LabeledMemoryWithFeedbackMetricsFeedbackMetrics"
    lookup_count: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value: str
        value = self.value

        embedding = self.embedding

        source_id: Union[None, str]
        source_id = self.source_id

        metadata = self.metadata.to_dict()

        memory_id = self.memory_id

        memory_version = self.memory_version

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        metrics = self.metrics.to_dict()

        label = self.label

        label_name: Union[None, str]
        label_name = self.label_name

        feedback_metrics = self.feedback_metrics.to_dict()

        lookup_count = self.lookup_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "value": value,
                "embedding": embedding,
                "source_id": source_id,
                "metadata": metadata,
                "memory_id": memory_id,
                "memory_version": memory_version,
                "created_at": created_at,
                "updated_at": updated_at,
                "metrics": metrics,
                "label": label,
                "label_name": label_name,
                "feedback_metrics": feedback_metrics,
                "lookup_count": lookup_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.labeled_memory_metrics import LabeledMemoryMetrics
        from ..models.labeled_memory_with_feedback_metrics_feedback_metrics import (
            LabeledMemoryWithFeedbackMetricsFeedbackMetrics,
        )
        from ..models.labeled_memory_with_feedback_metrics_metadata import LabeledMemoryWithFeedbackMetricsMetadata

        d = src_dict.copy()

        def _parse_value(data: object) -> str:
            return cast(str, data)

        value = _parse_value(d.pop("value"))

        embedding = cast(List[float], d.pop("embedding"))

        def _parse_source_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        source_id = _parse_source_id(d.pop("source_id"))

        metadata = LabeledMemoryWithFeedbackMetricsMetadata.from_dict(d.pop("metadata"))

        memory_id = d.pop("memory_id")

        memory_version = d.pop("memory_version")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        metrics = LabeledMemoryMetrics.from_dict(d.pop("metrics"))

        label = d.pop("label")

        def _parse_label_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        label_name = _parse_label_name(d.pop("label_name"))

        feedback_metrics = LabeledMemoryWithFeedbackMetricsFeedbackMetrics.from_dict(d.pop("feedback_metrics"))

        lookup_count = d.pop("lookup_count")

        labeled_memory_with_feedback_metrics = cls(
            value=value,
            embedding=embedding,
            source_id=source_id,
            metadata=metadata,
            memory_id=memory_id,
            memory_version=memory_version,
            created_at=created_at,
            updated_at=updated_at,
            metrics=metrics,
            label=label,
            label_name=label_name,
            feedback_metrics=feedback_metrics,
            lookup_count=lookup_count,
        )

        labeled_memory_with_feedback_metrics.additional_properties = d
        return labeled_memory_with_feedback_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.embedding_model_result import EmbeddingModelResult


T = TypeVar("T", bound="EmbeddingEvaluationResult")


@_attrs_define
class EmbeddingEvaluationResult:
    """
    Attributes:
        evaluation_results (Union[Unset, List['EmbeddingModelResult']]):
    """

    evaluation_results: Union[Unset, List["EmbeddingModelResult"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        evaluation_results: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.evaluation_results, Unset):
            evaluation_results = []
            for evaluation_results_item_data in self.evaluation_results:
                evaluation_results_item = evaluation_results_item_data.to_dict()
                evaluation_results.append(evaluation_results_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if evaluation_results is not UNSET:
            field_dict["evaluation_results"] = evaluation_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.embedding_model_result import EmbeddingModelResult

        d = src_dict.copy()
        evaluation_results = []
        _evaluation_results = d.pop("evaluation_results", UNSET)
        for evaluation_results_item_data in _evaluation_results or []:
            evaluation_results_item = EmbeddingModelResult.from_dict(evaluation_results_item_data)

            evaluation_results.append(evaluation_results_item)

        embedding_evaluation_result = cls(
            evaluation_results=evaluation_results,
        )

        embedding_evaluation_result.additional_properties = d
        return embedding_evaluation_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EmbeddingFinetuningMethod(str, Enum):
    BATCH_TRIPLET_LOSS = "batch_triplet_loss"
    CLASSIFICATION = "classification"

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import Any, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.pretrained_embedding_model_name import PretrainedEmbeddingModelName

T = TypeVar("T", bound="PretrainedEmbeddingModelMetadata")


@_attrs_define
class PretrainedEmbeddingModelMetadata:
    """
    Attributes:
        embedding_dim (int):
        max_seq_length (int):
        uses_context (bool):
        name (PretrainedEmbeddingModelName): Names of pretrained embedding models that are supported by OrcaCloud
    """

    embedding_dim: int
    max_seq_length: int
    uses_context: bool
    name: PretrainedEmbeddingModelName
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        embedding_dim = self.embedding_dim

        max_seq_length = self.max_seq_length

        uses_context = self.uses_context

        name = self.name.value if isinstance(self.name, Enum) else self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "embedding_dim": embedding_dim,
                "max_seq_length": max_seq_length,
                "uses_context": uses_context,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        embedding_dim = d.pop("embedding_dim")

        max_seq_length = d.pop("max_seq_length")

        uses_context = d.pop("uses_context")

        name = PretrainedEmbeddingModelName(d.pop("name"))

        pretrained_embedding_model_metadata = cls(
            embedding_dim=embedding_dim,
            max_seq_length=max_seq_length,
            uses_context=uses_context,
            name=name,
        )

        pretrained_embedding_model_metadata.additional_properties = d
        return pretrained_embedding_model_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
